from core import world
worldmod = world
from core.agents import *
import random

import pygame

class Sky(Agent):
    def __init__(self):
        super(Sky,self).__init__()
        self.color = [20,20,100]
    def update(self,world):
        t = world.time
        if t>5:
            t = 10-t
        r = g = 20+int(30*(t))
        b = 100+int(40*t)
        r = max(0,min(r,255))
        g = max(0,min(g,255))
        b = max(0,min(b,255))
        self.color = [r,g,b]
    def draw(self,engine):
        engine.surface.fill(self.color)
    def sortable(self):
        return 0
        
class Compound(Agent):
    def sortable(self):
        return 200
    
class Man(Agent):
    facing = 1
    def update(self,world):
        self.hotspot = [17,29]
        if self.facing == 1:
            self.load("core/ruthosaurus/man.png")
            self.pos[0]+=1
            if self.pos[0]>340:
                self.delete()
        elif self.facing == -1:
            self.load("core/ruthosaurus/man.png.flipped")
            self.pos[0]-=1
            if self.pos[0]<-20:
                self.delete()

class FlyingMan(Agent):
    food = True
    def update(self,world):
        self.rot += 1
        if self.rot>360:
            self.rot = 0
        self.pos[1]-=3
        if self.pos[1]<-40:
            self.delete()
    def sortable(self):
        return 500
        
class Hunger(TextAgent):
    chompy = None
    def update(self,world):
        self.lines = ["Hunger: %s"%int(self.chompy.hunger)]
    def sortable(self):
        return 501
        
class Chompy(Agent):
    def __init__(self):
        super(Chompy,self).__init__()
        self.dir = -1
        self.closed_right = pygame.image.load("core/ruthosaurus/rex_closed.png")
        self.closed_left = pygame.transform.flip(self.closed_right,1,0)
        self.closed_front = pygame.image.load("core/ruthosaurus/rex_front.png")
        self.closed = {-1:self.closed_left,1:self.closed_right,0:self.closed_front}
        self.open_right = pygame.image.load("core/ruthosaurus/rex_open.png")
        self.open_left = pygame.transform.flip(self.open_right,1,0)
        self.open_front = pygame.image.load("core/ruthosaurus/rex_front_open.png")
        self.open = {-1:self.open_left,1:self.open_right,0:self.open_front}
        self.sleep_right = pygame.image.load("core/ruthosaurus/rex_sleep.png")
        self.sleep_left = pygame.transform.flip(self.sleep_right,1,0)
        self.sleep = {-1:self.sleep_left,1:self.sleep_right,0:self.sleep_right}
        for d in [self.open,self.closed,self.sleep]:
            for x in d.values():
                x.set_colorkey([255,0,255])
        self.pos = [160,84]
        self.hotspot = [32,64]
        self.surface = self.closed_right
        self.mouth_open = False
        self.facing = 0
        self.sleeping = False
        self.interest = "outside"
        self.interest_level = 200
        self.interest_point = None
        self.activity = "stare"
        self.cycle_dir = 1
        self.next_roar = 250
        self.speed = 1
        self.hunger = 100
    def near(self,world):
        if self.facing == 0:
            spot = [self.pos[0],self.pos[1]-32]
        elif self.facing == 1:
            spot = [self.pos[0]+32,self.pos[1]-32]
        elif self.facing == -1:
            spot = [self.pos[0]-32,self.pos[1]-32]
        l = []
        for o in world.objects:
            r = o.rect()
            if r.collidepoint(spot):
                l.append(o)
        return l
    def food_near(self,world):
        return [x for x in self.near(world) if hasattr(x,"food")]
    def update(self,world):
        self.hunger -= 0.05
        if self.hunger < 0:
            world.replace(worldmod.GameOverWorld(world.engine,["Your dinosaur got starving and","broke out of the pen."]))
        self.food_near_list = self.food_near(world)
        self.cinterest(world)
        self.brain(world)
        self.move(world)
        self.update_art()
    def cinterest(self,world):
        self.last_interest = self.interest
        self.interest_level -= 1
        if self.interest_level < 0:
            self.interest_point = None
            interests = ["outside","inside","excercize"]
            if random.randint(0,1000)<998:
                self.interest = random.choice(interests)
            self.interest_level = random.randint(200,500)
        self.next_roar -= 1
        if self.next_roar < 0:
            self.next_roar = random.randint(250,650)
            self.interest = "makenoise"
            self.interest_level = 60
            self.roar_state = 0
        if world.time<2 or world.time>9:
            self.interest = "sleep"
        if self.food_near_list:
            self.interest = "eating"
            if self.last_interest != "eating":
                self.interest_level = 300
                self.eating_state = 0
    def brain(self,world):
        if self.interest == "outside":
            self.speed = 1
            if not self.interest_point:
                left = max(50,self.pos[0]-50)
                right = min(270,self.pos[0]+50)
                p = random.randint(left,right)
                self.interest_point = p
            d = self.pos[0]-self.interest_point
            dmax = abs(d)
            if abs(d)>3:
                if d>0:
                    self.activity = "left"
                elif d<0:
                    self.activity = "right"
            else:
                self.activity = "stare"
        elif self.interest == "inside":
            self.speed = 0.5
            if not self.interest_point:
                left = int(max(50,self.pos[0]-50))
                right = int(min(270,self.pos[0]+50))
                p = random.randint(left,right)
                self.interest_point = p
            d = self.pos[0]-self.interest_point
            dmax = abs(d)
            if abs(d)>3:
                if d>0:
                    self.activity = "left"
                elif d<0:
                    self.activity = "right"
            else:
                self.activity = "staredown"
        elif self.interest == "excercize":
            if self.pos[0]<20:
                self.facing = 1
            elif self.pos[0]>300:
                self.facing = -1
            elif self.facing == 0:
                self.facing = random.choice([-1,1])
            if self.facing == 1:
                self.activity = "right"
            elif self.facing == -1:
                self.activity = "left"
            self.speed = random.choice([2,3,4])
        elif self.interest == "makenoise":
            self.activity = "roar"
        elif self.interest == "eating":
            self.activity = "eating"
        elif self.interest == "sleep":
            self.activity = "sleep"
            if world.time>2:
                self.interest_level = 0
    def cycle_y(self,amt=0,min=80,max=95):
        self.pos[1]+=amt*self.cycle_dir
        if self.pos[1]>max:
            self.cycle_dir = -1
            self.pos[1]=max
        if self.pos[1]<min:
            self.cycle_dir = 1
            self.pos[1]=min
    def move(self,world):
        if self.activity == "stare":
            self.facing = 0
            self.cycle_y(random.random()*0.1)
        if self.activity == "staredown":
            self.cycle_y(random.random()*0.1,90,100)
        if self.activity == "left":
            self.facing = -1
            self.pos[0]-=self.speed
            self.cycle_y(0.5)
            if self.pos[0]<5:
                self.pos[0]=5
                self.interest_level -= 20
        if self.activity == "right":
            self.facing = 1
            self.pos[0]+=self.speed
            self.cycle_y(0.5)
            if self.pos[0]>315:
                self.pos[0]=315
                self.interest_level -= 20
        if self.activity == "roar":
            if self.roar_state == 0:
                self.engine.sound("core/ruthosaurus/roar1.ogg")
                self.mouth_open = True
            self.roar_state += 1
            if self.roar_state > 30:
                self.mouth_open = False
            if self.roar_state < 60:
                self.interest_level = 50
        if self.activity == "eating":
            self.eating_state += 1
            if self.eating_state == 1:
                self.mouth_open = True
                world.engine.sound("core/ruthosaurus/bite.ogg")
            elif self.eating_state == 30:
                self.mouth_open = False
            elif self.eating_state == 60:
                self.eating_state = 0
                self.mouth_open = False
            for o in self.food_near_list:
                o.delete()
                self.hunger +=2
                if self.hunger>100:
                    self.hunger = 100
        if self.activity == "sleep":
            self.sleeping = True
        else:
            self.sleeping = False
    def update_art(self):
        if self.sleeping:
            self.surface = self.sleep[self.facing]
        elif self.mouth_open:
            self.surface = self.open[self.facing]
        else:
            self.surface = self.closed[self.facing]
            
class SaurWorld(world.World):
    time = 5
    def input(self,controller):
        #~ if pygame.key.get_pressed()[pygame.K_LEFT]:
            #~ self.chompy.pos[0]-=1
        #~ if pygame.key.get_pressed()[pygame.K_RIGHT]:
            #~ self.chompy.pos[0]+=1
        #~ if self.chompy.pos[0]<16:
            #~ self.chompy.pos[0] = 16
        #~ if self.chompy.pos[0]>310:
            #~ self.chompy.pos[0]=310
        super(SaurWorld,self).input(controller)
    def update(self):
        self.time += 0.005
        if self.time>10:
            self.time = 0
        man_chance = 0.001
        if self.time>3 and self.time < 7:
            man_chance = 0.05
        if random.random()<man_chance:
            for i in range(random.randint(1,3)):
                dir = random.choice([-1,1])
                if dir == -1:
                    p = [random.randint(340,380),random.randint(197,238)]
                elif dir == 1:
                    p = [random.randint(-60,-20),random.randint(197,238)]
                m = Man(pos=p)
                m.facing = dir
                self.add(m)
        super(SaurWorld,self).update()
    def input(self,controller):
        if controller.mouse_clicked:
            pos = list(controller.mouse_clicked.pos)
            pos[0]=int(pos[0]*0.5)
            pos[1]=int(pos[1]*0.5)
            for o in self.objects:
                if not isinstance(o,Man):
                    continue
                if o.rect().collidepoint(pos):
                    o.delete()
                    self.add(FlyingMan("core/ruthosaurus/man.png",o.pos[:]))
        
def make_world(engine):
    """This makes the starting world"""
    w = SaurWorld(engine)
    w.add(Sky())
    c = Chompy()
    w.add(c)
    w.add(Compound("core/ruthosaurus/pen.png",[0,0]))
    h = Hunger([],pos=[0,0])
    h.chompy = c
    w.add(h)
    return w
